import { from, Observable, Subscription } from 'rxjs';
import { concatMap as rxConcatMap, mergeMap as rxMergeMap, switchMap as rxSwitchMap } from 'rxjs/operators';
export const create = (subscribe) => new Observable(subscriber => {
    const abortController = new AbortController();
    const subscription = new Subscription();
    const teardown = subscribe && subscribe(subscriber, abortController.signal);
    subscription.add(teardown);
    subscription.add(() => abortController.abort());
    return subscription;
});
export const defer = (factory) => create((subscriber, signal) => from(factory(signal)).subscribe(subscriber));
export const toPromise = (observable, signal) => new Promise((resolve, reject) => {
    let value;
    const subscription = observable.subscribe(val => {
        value = val;
    }, reject, () => {
        resolve(value);
    });
    if (signal) {
        signal.addEventListener('abort', () => {
            subscription.unsubscribe();
            const error = new Error('Aborted');
            error.name = 'AbortError';
            reject(error);
        });
    }
});
export const forEach = (source, next, signal) => new Promise((resolve, reject) => {
    // Must be declared in a separate statement to avoid a RefernceError when
    // accessing subscription below in the closure due to Temporal Dead Zone.
    let subscription;
    subscription = source.subscribe(value => {
        try {
            next(value);
        }
        catch (err) {
            reject(err);
            if (subscription) {
                subscription.unsubscribe();
            }
        }
    }, reject, resolve);
    if (signal) {
        signal.addEventListener('abort', () => {
            subscription.unsubscribe();
            const error = new Error('Aborted');
            error.name = 'AbortError';
            reject(error);
        });
    }
});
export const switchMap = (project) => source => source.pipe(rxSwitchMap((value, index) => defer(abortSignal => project(value, index, abortSignal))));
export const concatMap = (project) => source => source.pipe(rxConcatMap((value, index) => defer(abortSignal => project(value, index, abortSignal))));
export const mergeMap = (project) => source => source.pipe(rxMergeMap((value, index) => defer(abortSignal => project(value, index, abortSignal))));
//# sourceMappingURL=index.js.map